@@ -23,7 +23,6 @@ import uuid
 import errno
 import re
 from contextlib import contextmanager
-from swift import gettext_ as _
 
 from eventlet import GreenPool, sleep, Timeout
 from eventlet.green import subprocess
@@ -176,7 +175,7 @@ class ReplConnection(BufferedHTTPConnection):
         except (Exception, Timeout):
             self.close()
             self.logger.exception(
-                _('ERROR reading HTTP response from %s'), self.node)
+                'ERROR reading HTTP response from %s', self.node)
             return None
 
 
@@ -241,6 +240,12 @@ class Replicator(Daemon):
         self.handoffs_only = config_true_value(conf.get('handoffs_only', 'no'))
         self.handoff_delete = config_auto_int_value(
             conf.get('handoff_delete', 'auto'), 0)
+        if self.handoff_delete >= self.ring.replica_count:
+            self.logger.warning(
+                'handoff_delete=%d is too high to have an effect on a ring '
+                'with replica count %d. Disabling.',
+                self.handoff_delete, self.ring.replica_count)
+            self.handoff_delete = 0
 
     def _zero_stats(self):
         """Zero out the stats."""
@@ -254,15 +259,15 @@ class Replicator(Daemon):
         """Report the current stats to the logs."""
         now = time.time()
         self.logger.info(
-            _('Attempted to replicate %(count)d dbs in %(time).5f seconds '
-              '(%(rate).5f/s)'),
+            'Attempted to replicate %(count)d dbs in %(time).5f seconds '
+            '(%(rate).5f/s)',
             {'count': self.stats['attempted'],
              'time': now - self.stats['start'],
              'rate': self.stats['attempted'] /
                 (now - self.stats['start'] + 0.0000001)})
-        self.logger.info(_('Removed %(remove)d dbs') % self.stats)
-        self.logger.info(_('%(success)s successes, %(failure)s failures')
-                         % self.stats)
+        self.logger.info('Removed %(remove)d dbs', self.stats)
+        self.logger.info('%(success)s successes, %(failure)s failures',
+                         self.stats)
         dump_recon_cache(
             {'replication_stats': self.stats,
              'replication_time': now - self.stats['start'],
@@ -308,7 +313,7 @@ class Replicator(Daemon):
         proc = subprocess.Popen(popen_args)
         proc.communicate()
         if proc.returncode != 0:
-            self.logger.error(_('ERROR rsync failed with %(code)s: %(args)s'),
+            self.logger.error('ERROR rsync failed with %(code)s: %(args)s',
                               {'code': proc.returncode, 'args': popen_args})
         return proc.returncode == 0
 
@@ -625,10 +630,10 @@ class Replicator(Daemon):
                     'replicate out and remove.' % (object_file, name, bpart))
         except (Exception, Timeout) as e:
             if 'no such table' in str(e):
-                self.logger.error(_('Quarantining DB %s'), object_file)
+                self.logger.error('Quarantining DB %s', object_file)
                 quarantine_db(broker.db_file, broker.db_type)
             else:
-                self.logger.exception(_('ERROR reading db %s'), object_file)
+                self.logger.exception('ERROR reading db %s', object_file)
             nodes = self.ring.get_part_nodes(int(partition))
             self._add_failure_stats([(failure_dev['replication_ip'],
                                       failure_dev['device'])
@@ -680,13 +685,13 @@ class Replicator(Daemon):
                     repl_nodes.append(next(more_nodes))
                 except StopIteration:
                     self.logger.error(
-                        _('ERROR There are not enough handoff nodes to reach '
-                          'replica count for partition %s'),
+                        'ERROR There are not enough handoff nodes to reach '
+                        'replica count for partition %s',
                         partition)
-                self.logger.error(_('ERROR Remote drive not mounted %s'), node)
+                self.logger.error('ERROR Remote drive not mounted %s', node)
             except (Exception, Timeout):
-                self.logger.exception(_('ERROR syncing %(file)s with node'
-                                        ' %(node)s'),
+                self.logger.exception('ERROR syncing %(file)s with node'
+                                      ' %(node)s',
                                       {'file': object_file, 'node': node})
             if not success:
                 failure_devs_info.add((node['replication_ip'], node['device']))
@@ -785,7 +790,7 @@ class Replicator(Daemon):
         dirs = []
         ips = whataremyips(self.bind_ip)
         if not ips:
-            self.logger.error(_('ERROR Failed to get my own IPs?'))
+            self.logger.error('ERROR Failed to get my own IPs?')
             return
 
         if self.handoffs_only or self.handoff_delete:
@@ -830,12 +835,12 @@ class Replicator(Daemon):
             self.logger.error("Can't find itself %s with port %s in ring "
                               "file, not replicating",
                               ", ".join(ips), self.port)
-        self.logger.info(_('Beginning replication run'))
+        self.logger.info('Beginning replication run')
         for part, object_file, node_id in self.roundrobin_datadirs(dirs):
             self.cpool.spawn_n(
                 self._replicate_object, part, object_file, node_id)
         self.cpool.waitall()
-        self.logger.info(_('Replication run OVER'))
+        self.logger.info('Replication run OVER')
         if self.handoffs_only or self.handoff_delete:
             self.logger.warning(
                 'Finished replication pass with handoffs_only and/or '
@@ -853,7 +858,7 @@ class Replicator(Daemon):
             try:
                 self.run_once()
             except (Exception, Timeout):
-                self.logger.exception(_('ERROR trying to replicate'))
+                self.logger.exception('ERROR trying to replicate')
             elapsed = time.time() - begin
             if elapsed < self.interval:
                 sleep(self.interval - elapsed)
@@ -957,7 +962,7 @@ class ReplicatorRpc(object):
                 info = self._get_synced_replication_info(broker, remote_info)
             except (Exception, Timeout) as e:
                 if 'no such table' in str(e):
-                    self.logger.error(_("Quarantining DB %s"), broker)
+                    self.logger.error("Quarantining DB %s", broker)
                     quarantine_db(broker.db_file, broker.db_type)
                     return HTTPNotFound()
                 raise
