@@ -17,7 +17,7 @@ import logging
 import os
 
 from swift.common.middleware.crypto import keymaster
-from swift.common.utils import LogLevelFilter
+from swift.common.utils import LogLevelFilter, load_multikey_opts
 
 from kmip.pie.client import ProxyKmipClient
 
@@ -98,18 +98,21 @@ example::
 """
 
 
-class KmipKeyMaster(keymaster.KeyMaster):
+class KmipKeyMaster(keymaster.BaseKeyMaster):
     log_route = 'kmip_keymaster'
     keymaster_opts = ('host', 'port', 'certfile', 'keyfile',
                       'ca_certs', 'username', 'password',
                       'active_root_secret_id', 'key_id*')
     keymaster_conf_section = 'kmip_keymaster'
 
-    def _get_root_secret(self, conf):
+    def _load_keymaster_config_file(self, conf):
+        conf = super(KmipKeyMaster, self)._load_keymaster_config_file(conf)
         if self.keymaster_config_path:
             section = self.keymaster_conf_section
         else:
-            section = conf['__name__']
+            # __name__ is just the filter name, not the whole section name.
+            # Luckily, PasteDeploy only uses the one prefix for filters.
+            section = 'filter:' + conf['__name__']
 
         if os.path.isdir(conf['__file__']):
             raise ValueError(
@@ -117,6 +120,8 @@ class KmipKeyMaster(keymaster.KeyMaster):
                 'keymaster_config_path option in the proxy server config to '
                 'specify a config file.')
 
+        # Make sure we've got the kmip log handler set up before
+        # we instantiate a client
         kmip_logger = logging.getLogger('kmip')
         for handler in self.logger.logger.handlers:
             kmip_logger.addHandler(handler)
@@ -133,15 +138,17 @@ class KmipKeyMaster(keymaster.KeyMaster):
         ):
             logging.getLogger(name).addFilter(debug_filter)
 
-        multikey_opts = self._load_multikey_opts(conf, 'key_id')
-        if not multikey_opts:
-            raise ValueError('key_id option is required')
-        kmip_to_secret = {}
-        root_secrets = {}
-        with ProxyKmipClient(
+        self.proxy_kmip_client = ProxyKmipClient(
             config=section,
             config_file=conf['__file__']
-        ) as client:
+        )
+        return conf
+
+    def _get_root_secret(self, conf):
+        multikey_opts = load_multikey_opts(conf, 'key_id', allow_none_key=True)
+        kmip_to_secret = {}
+        root_secrets = {}
+        with self.proxy_kmip_client as client:
             for opt, secret_id, kmip_id in multikey_opts:
                 if kmip_id in kmip_to_secret:
                     # Save some round trips if there are multiple
