@@ -16,14 +16,14 @@
 from base64 import standard_b64encode as b64encode
 from base64 import standard_b64decode as b64decode
 
-import six
-from six.moves.urllib.parse import quote
+from urllib.parse import quote
 
 from swift.common import swob
 from swift.common.http import HTTP_OK
 from swift.common.middleware.versioned_writes.object_versioning import \
     DELETE_MARKER_CONTENT_TYPE
-from swift.common.utils import json, public, config_true_value, Timestamp
+from swift.common.utils import json, public, config_true_value, Timestamp, \
+    cap_length
 from swift.common.registry import get_swift_info
 
 from swift.common.middleware.s3api.controllers.base import Controller
@@ -144,9 +144,8 @@ class BucketController(Controller):
                 query['marker'] = swob.wsgi_to_str(req.params['start-after'])
             # continuation-token overrides start-after
             if 'continuation-token' in req.params:
-                decoded = b64decode(req.params['continuation-token'])
-                if not six.PY2:
-                    decoded = decoded.decode('utf8')
+                decoded = b64decode(
+                    req.params['continuation-token']).decode('utf8')
                 query['marker'] = decoded
             if 'fetch-owner' in req.params:
                 fetch_owner = config_true_value(req.params['fetch-owner'])
@@ -343,7 +342,12 @@ class BucketController(Controller):
 
         resp = req.get_response(self.app, query=query)
 
-        objects = json.loads(resp.body)
+        try:
+            objects = json.loads(resp.body)
+        except (TypeError, ValueError):
+            self.logger.error('Got non-JSON response trying to list %s: %r',
+                              req.path, cap_length(resp.body, 60))
+            raise
 
         is_truncated = max_keys > 0 and len(objects) > max_keys
         objects = objects[:max_keys]
