@@ -366,13 +366,12 @@ from swift.common.registry import register_swift_info
 from swift.common.request_helpers import SegmentedIterable, \
     get_sys_meta_prefix, update_etag_is_at_header, resolve_etag_is_at_header, \
     get_container_update_override_key, update_ignore_range_header, \
-    get_param, get_valid_part_num
+    get_param, get_valid_part_num, get_heartbeat_response_body
 from swift.common.constraints import check_utf8
 from swift.common.http import HTTP_NOT_FOUND, HTTP_UNAUTHORIZED
 from swift.common.wsgi import WSGIContext, make_subrequest, make_env, \
     make_pre_authed_request
-from swift.common.middleware.bulk import get_response_body, \
-    ACCEPTABLE_FORMATS, Bulk
+from swift.common.middleware.bulk import ACCEPTABLE_FORMATS, Bulk
 from swift.obj import expirer
 from swift.proxy.controllers.base import get_container_info
 
@@ -1527,7 +1526,7 @@ class StaticLargeObject(object):
                     start_response(err.status,
                                    [(h, v) for h, v in err.headers.items()
                                     if h.lower() != 'content-length'])
-                yield separator + get_response_body(
+                yield separator + get_heartbeat_response_body(
                     out_content_type, resp_dict, problem_segments, 'upload')
                 return
 
@@ -1554,7 +1553,7 @@ class StaticLargeObject(object):
                         err_body = err_body.decode('utf-8', errors='replace')
                     resp_dict['Response Body'] = err_body or '\n'.join(
                         RESPONSE_REASONS.get(err.status_int, ['']))
-                    yield separator + get_response_body(
+                    yield separator + get_heartbeat_response_body(
                         out_content_type, resp_dict, problem_segments,
                         'upload')
                 else:
@@ -1600,7 +1599,7 @@ class StaticLargeObject(object):
                 if isinstance(resp_body, bytes):
                     resp_body = resp_body.decode('utf-8')
                 resp_dict['Response Body'] = resp_body
-                yield separator + get_response_body(
+                yield separator + get_heartbeat_response_body(
                     out_content_type, resp_dict, [], 'upload')
             else:
                 for chunk in resp(req.environ, start_response):
