@@ -14,54 +14,107 @@
 # limitations under the License.
 
 import array
-import six.moves.cPickle as pickle
+
 import json
 from collections import defaultdict
-from gzip import GzipFile
 from os.path import getmtime
 import struct
 from time import time
 import os
-from io import BufferedReader
-from hashlib import md5
-from itertools import chain
-from tempfile import NamedTemporaryFile
+from itertools import chain, count
 import sys
 
-from six.moves import range
-
-from swift.common.exceptions import RingLoadError
-from swift.common.utils import hash_path, validate_configuration
+from swift.common.exceptions import RingLoadError, DevIdBytesTooSmall
+from swift.common.utils import hash_path, validate_configuration, md5
+from swift.common.ring.io import RingReader, RingWriter
 from swift.common.ring.utils import tiers_for_dev
 
 
+DEFAULT_RELOAD_TIME = 15
+RING_CODECS = {
+    1: {
+        "serialize": lambda ring_data, writer: ring_data.serialize_v1(writer),
+        "deserialize": lambda cls, reader, metadata_only, _include_devices:
+            cls.deserialize_v1(reader, metadata_only=metadata_only),
+    },
+    2: {
+        "serialize": lambda ring_data, writer: ring_data.serialize_v2(writer),
+        "deserialize": lambda cls, reader, metadata_only, include_devices:
+            cls.deserialize_v2(reader, metadata_only=metadata_only,
+                               include_devices=include_devices),
+    },
+}
+DEFAULT_RING_FORMAT_VERSION = 1
+
+
 def calc_replica_count(replica2part2dev_id):
+    if not replica2part2dev_id:
+        return 0
     base = len(replica2part2dev_id) - 1
     extra = 1.0 * len(replica2part2dev_id[-1]) / len(replica2part2dev_id[0])
     return base + extra
 
 
+def normalize_devices(devs):
+    # NOTE(akscram): Replication parameters like replication_ip
+    #                and replication_port are required for
+    #                replication process. An old replication
+    #                ring doesn't contain this parameters into
+    #                device.
+    for dev in devs:
+        if dev is None:
+            continue
+        if 'ip' in dev:
+            dev.setdefault('replication_ip', dev['ip'])
+        if 'port' in dev:
+            dev.setdefault('replication_port', dev['port'])
+
+
 class RingData(object):
     """Partitioned consistent hashing ring data (used for serialization)."""
 
     def __init__(self, replica2part2dev_id, devs, part_shift,
-                 next_part_power=None):
+                 next_part_power=None, version=None):
+        normalize_devices(devs)
         self.devs = devs
+        for i, part2dev_id in enumerate(replica2part2dev_id):
+            if not isinstance(part2dev_id, array.array):
+                replica2part2dev_id[i] = array.array('H', part2dev_id)
         self._replica2part2dev_id = replica2part2dev_id
         self._part_shift = part_shift
         self.next_part_power = next_part_power
-
-        for dev in self.devs:
-            if dev is not None:
-                dev.setdefault("region", 1)
+        self.version = version
+        self.format_version = None
+        self.size = self.raw_size = None
+        # Next two are used when replica2part2dev is empty
+        self._dev_id_bytes = 2
+        self._replica_count = 0
+        self._num_devs = sum(1 if dev is not None else 0 for dev in self.devs)
 
     @property
     def replica_count(self):
         """Number of replicas (full or partial) used in the ring."""
-        return calc_replica_count(self._replica2part2dev_id)
+        if self._replica2part2dev_id:
+            return calc_replica_count(self._replica2part2dev_id)
+        else:
+            return self._replica_count
+
+    @property
+    def part_power(self):
+        return 32 - self._part_shift
+
+    @property
+    def dev_id_bytes(self):
+        if self._replica2part2dev_id:
+            # There's an assumption that these will all have the same itemsize,
+            # but just in case...
+            return max(part2dev_id.itemsize
+                       for part2dev_id in self._replica2part2dev_id)
+        else:
+            return self._dev_id_bytes
 
     @classmethod
-    def deserialize_v1(cls, gz_file, metadata_only=False):
+    def deserialize_v1(cls, reader, metadata_only=False):
         """
         Deserialize a v1 ring file into a dictionary with `devs`, `part_shift`,
         and `replica2part2dev_id` keys.
@@ -70,15 +123,16 @@ class RingData(object):
         `replica2part2dev_id` is not loaded and that key in the returned
         dictionary just has the value `[]`.
 
-        :param file gz_file: An opened file-like object which has already
-                             consumed the 6 bytes of magic and version.
+        :param RingReader reader: An opened RingReader at the start of the file
         :param bool metadata_only: If True, only load `devs` and `part_shift`
         :returns: A dict containing `devs`, `part_shift`, and
                   `replica2part2dev_id`
         """
+        magic = reader.read(6)
+        if magic != b'R1NG\x00\x01':
+            raise ValueError('unexpected magic: %r' % magic)
 
-        json_len, = struct.unpack('!I', gz_file.read(4))
-        ring_dict = json.loads(gz_file.read(json_len).decode('ascii'))
+        ring_dict = json.loads(reader.read_blob('!I'))
         ring_dict['replica2part2dev_id'] = []
 
         if metadata_only:
@@ -88,7 +142,7 @@ class RingData(object):
 
         partition_count = 1 << (32 - ring_dict['part_shift'])
         for x in range(ring_dict['replica_count']):
-            part2dev = array.array('H', gz_file.read(2 * partition_count))
+            part2dev = array.array('H', reader.read(2 * partition_count))
             if byteswap:
                 part2dev.byteswap()
             ring_dict['replica2part2dev_id'].append(part2dev)
@@ -96,7 +150,49 @@ class RingData(object):
         return ring_dict
 
     @classmethod
-    def load(cls, filename, metadata_only=False):
+    def deserialize_v2(cls, reader, metadata_only=False, include_devices=True):
+        """
+        Deserialize a v2 ring file into a dictionary with ``devs``,
+        ``part_shift``, and ``replica2part2dev_id`` keys.
+
+        If the optional kwarg ``metadata_only`` is True, then the
+        ``replica2part2dev_id`` is not loaded and that key in the returned
+        dictionary just has the value ``[]``.
+
+        If the optional kwarg ``include_devices`` is False, then the ``devs``
+        list is not loaded and that key in the returned dictionary just has
+        the value ``[]``.
+
+        :param RingReader reader: An opened RingReader which has already
+                                  loaded up the index at the end of the file.
+        :param bool metadata_only: If True, skip loading
+                                   ``replica2part2dev_id``
+        :param bool include_devices: If False, skip loading ``devs``
+        :returns: A dict containing ``devs``, ``part_shift``,
+                  ``dev_id_bytes``, and ``replica2part2dev_id``
+        """
+
+        ring_dict = json.loads(reader.read_section('swift/ring/metadata'))
+        ring_dict['replica2part2dev_id'] = []
+        ring_dict['devs'] = []
+
+        if include_devices:
+            ring_dict['devs'] = json.loads(
+                reader.read_section('swift/ring/devices'))
+
+        if metadata_only:
+            return ring_dict
+
+        partition_count = 1 << (32 - ring_dict['part_shift'])
+
+        with reader.open_section('swift/ring/assignments') as section:
+            ring_dict['replica2part2dev_id'] = section.read_ring_table(
+                ring_dict['dev_id_bytes'], partition_count)
+
+        return ring_dict
+
+    @classmethod
+    def load(cls, filename, metadata_only=False, include_devices=True):
         """
         Load ring data from a file.
 
@@ -104,32 +200,37 @@ class RingData(object):
         :param bool metadata_only: If True, only load `devs` and `part_shift`.
         :returns: A RingData instance containing the loaded data.
         """
-        gz_file = BufferedReader(GzipFile(filename, 'rb'))
+        with RingReader.open(filename) as reader:
+            if reader.version not in RING_CODECS:
+                raise Exception('Unknown ring format version %d for %r' % (
+                                reader.version, filename))
+            ring_data = RING_CODECS[reader.version]['deserialize'](
+                cls, reader, metadata_only, include_devices)
 
-        # See if the file is in the new format
-        magic = gz_file.read(4)
-        if magic == b'R1NG':
-            format_version, = struct.unpack('!H', gz_file.read(2))
-            if format_version == 1:
-                ring_data = cls.deserialize_v1(
-                    gz_file, metadata_only=metadata_only)
-            else:
-                raise Exception('Unknown ring format version %d' %
-                                format_version)
-        else:
-            # Assume old-style pickled ring
-            gz_file.seek(0)
-            ring_data = pickle.load(gz_file)
-
-        if not hasattr(ring_data, 'devs'):
-            ring_data = RingData(ring_data['replica2part2dev_id'],
-                                 ring_data['devs'], ring_data['part_shift'],
-                                 ring_data.get('next_part_power'))
+        ring_data = cls.from_dict(ring_data)
+        ring_data.format_version = reader.version
+        for attr in ('size', 'raw_size'):
+            setattr(ring_data, attr, getattr(reader, attr))
         return ring_data
 
-    def serialize_v1(self, file_obj):
+    @classmethod
+    def from_dict(cls, ring_data):
+        ring = cls(ring_data['replica2part2dev_id'],
+                   ring_data['devs'], ring_data['part_shift'],
+                   ring_data.get('next_part_power'),
+                   ring_data.get('version'))
+        # For loading with metadata_only=True
+        if 'replica_count' in ring_data:
+            ring._replica_count = ring_data['replica_count']
+        # dev_id_bytes only written down in v2 and above
+        ring._dev_id_bytes = ring_data.get('dev_id_bytes', 2)
+        return ring
+
+    def serialize_v1(self, writer):
+        if self.dev_id_bytes != 2:
+            raise DevIdBytesTooSmall('Ring v1 only supports 2-byte dev IDs')
         # Write out new-style serialization magic and version:
-        file_obj.write(struct.pack('!4sH', b'R1NG', 1))
+        writer.write_magic(version=1)
         ring = self.to_dict()
 
         # Only include next_part_power if it is set in the
@@ -138,6 +239,9 @@ class RingData(object):
                  'replica_count': len(ring['replica2part2dev_id']),
                  'byteorder': sys.byteorder}
 
+        if ring['version'] is not None:
+            _text['version'] = ring['version']
+
         next_part_power = ring.get('next_part_power')
         if next_part_power is not None:
             _text['next_part_power'] = next_part_power
@@ -145,37 +249,63 @@ class RingData(object):
         json_text = json.dumps(_text, sort_keys=True,
                                ensure_ascii=True).encode('ascii')
         json_len = len(json_text)
-        file_obj.write(struct.pack('!I', json_len))
-        file_obj.write(json_text)
+        writer.write(struct.pack('!I', json_len))
+        writer.write(json_text)
         for part2dev_id in ring['replica2part2dev_id']:
-            file_obj.write(part2dev_id.tostring())
+            part2dev_id.tofile(writer)
 
-    def save(self, filename, mtime=1300507380.0):
+    def serialize_v2(self, writer):
+        writer.write_magic(version=2)
+        ring = self.to_dict()
+
+        # Only include next_part_power if it is set in the
+        # builder, otherwise just ignore it
+        _text = {
+            'part_shift': ring['part_shift'],
+            'dev_id_bytes': ring['dev_id_bytes'],
+            'replica_count': calc_replica_count(ring['replica2part2dev_id']),
+            'version': ring['version']}
+
+        next_part_power = ring.get('next_part_power')
+        if next_part_power is not None:
+            _text['next_part_power'] = next_part_power
+
+        with writer.section('swift/ring/metadata'):
+            writer.write_json(_text)
+
+        with writer.section('swift/ring/devices'):
+            writer.write_json(ring['devs'])
+
+        with writer.section('swift/ring/assignments'):
+            writer.write_ring_table(ring['replica2part2dev_id'])
+
+    def save(self, filename, mtime=1300507380.0,
+             format_version=DEFAULT_RING_FORMAT_VERSION):
         """
         Serialize this RingData instance to disk.
 
         :param filename: File into which this instance should be serialized.
         :param mtime: time used to override mtime for gzip, default or None
                       if the caller wants to include time
+        :param format_version: one of 0, 1, or 2. Older versions are retained
+                               for the sake of clusters on older versions
         """
+        if format_version not in RING_CODECS:
+            raise ValueError("format_version must be one of %r" % (tuple(
+                RING_CODECS.keys()),))
         # Override the timestamp so that the same ring data creates
         # the same bytes on disk. This makes a checksum comparison a
         # good way to see if two rings are identical.
-        tempf = NamedTemporaryFile(dir=".", prefix=filename, delete=False)
-        gz_file = GzipFile(filename, mode='wb', fileobj=tempf, mtime=mtime)
-        self.serialize_v1(gz_file)
-        gz_file.close()
-        tempf.flush()
-        os.fsync(tempf.fileno())
-        tempf.close()
-        os.chmod(tempf.name, 0o644)
-        os.rename(tempf.name, filename)
+        with RingWriter.open(filename, mtime) as writer:
+            RING_CODECS[format_version]['serialize'](self, writer)
 
     def to_dict(self):
         return {'devs': self.devs,
                 'replica2part2dev_id': self._replica2part2dev_id,
                 'part_shift': self._part_shift,
-                'next_part_power': self.next_part_power}
+                'next_part_power': self.next_part_power,
+                'dev_id_bytes': self.dev_id_bytes,
+                'version': self.version}
 
 
 class Ring(object):
@@ -190,7 +320,7 @@ class Ring(object):
     :raises RingLoadError: if the loaded ring data violates its constraint
     """
 
-    def __init__(self, serialized_path, reload_time=15, ring_name=None,
+    def __init__(self, serialized_path, reload_time=None, ring_name=None,
                  validation_hook=lambda ring_data: None):
         # can't use the ring unless HASH_PATH_SUFFIX is set
         validate_configuration()
@@ -199,7 +329,8 @@ class Ring(object):
                                                 ring_name + '.ring.gz')
         else:
             self.serialized_path = os.path.join(serialized_path)
-        self.reload_time = reload_time
+        self.reload_time = (DEFAULT_RELOAD_TIME if reload_time is None
+                            else reload_time)
         self._validation_hook = validation_hook
         self._reload(force=True)
 
@@ -220,60 +351,82 @@ class Ring(object):
 
             self._mtime = getmtime(self.serialized_path)
             self._devs = ring_data.devs
-            # NOTE(akscram): Replication parameters like replication_ip
-            #                and replication_port are required for
-            #                replication process. An old replication
-            #                ring doesn't contain this parameters into
-            #                device. Old-style pickled rings won't have
-            #                region information.
-            for dev in self._devs:
-                if dev:
-                    dev.setdefault('region', 1)
-                    if 'ip' in dev:
-                        dev.setdefault('replication_ip', dev['ip'])
-                    if 'port' in dev:
-                        dev.setdefault('replication_port', dev['port'])
-
+            self._dev_id_bytes = ring_data._dev_id_bytes
             self._replica2part2dev_id = ring_data._replica2part2dev_id
             self._part_shift = ring_data._part_shift
             self._rebuild_tier_data()
-
-            # Do this now, when we know the data has changed, rather than
-            # doing it on every call to get_more_nodes().
-            #
-            # Since this is to speed up the finding of handoffs, we only
-            # consider devices with at least one partition assigned. This
-            # way, a region, zone, or server with no partitions assigned
-            # does not count toward our totals, thereby keeping the early
-            # bailouts in get_more_nodes() working.
-            dev_ids_with_parts = set()
-            for part2dev_id in self._replica2part2dev_id:
-                for dev_id in part2dev_id:
-                    dev_ids_with_parts.add(dev_id)
-
-            regions = set()
-            zones = set()
-            ips = set()
-            self._num_devs = 0
-            for dev in self._devs:
-                if dev and dev['id'] in dev_ids_with_parts:
-                    regions.add(dev['region'])
-                    zones.add((dev['region'], dev['zone']))
-                    ips.add((dev['region'], dev['zone'], dev['ip']))
-                    self._num_devs += 1
-            self._num_regions = len(regions)
-            self._num_zones = len(zones)
-            self._num_ips = len(ips)
+            self._update_bookkeeping()
             self._next_part_power = ring_data.next_part_power
+            self._version = ring_data.version
+            self._size = ring_data.size
+            self._raw_size = ring_data.raw_size
+
+    def _update_bookkeeping(self):
+        # Do this now, when we know the data has changed, rather than
+        # doing it on every call to get_more_nodes().
+        #
+        # Since this is to speed up the finding of handoffs, we only
+        # consider devices with at least one partition assigned. This
+        # way, a region, zone, or server with no partitions assigned
+        # does not count toward our totals, thereby keeping the early
+        # bailouts in get_more_nodes() working.
+        dev_ids_with_parts = set()
+        for part2dev_id in self._replica2part2dev_id:
+            for dev_id in part2dev_id:
+                dev_ids_with_parts.add(dev_id)
+        regions = set()
+        zones = set()
+        ips = set()
+        self._num_devs = 0
+        self._num_assigned_devs = 0
+        self._num_weighted_devs = 0
+        for dev in self._devs:
+            if dev is None:
+                continue
+            self._num_devs += 1
+            if dev.get('weight', 0) > 0:
+                self._num_weighted_devs += 1
+            if dev['id'] in dev_ids_with_parts:
+                regions.add(dev['region'])
+                zones.add((dev['region'], dev['zone']))
+                ips.add((dev['region'], dev['zone'], dev['ip']))
+                self._num_assigned_devs += 1
+        self._num_regions = len(regions)
+        self._num_zones = len(zones)
+        self._num_ips = len(ips)
+
+    @property
+    def dev_id_bytes(self):
+        if self._replica2part2dev_id:
+            # There's an assumption that these will all have the same itemsize,
+            # but just in case...
+            return max(part2dev_id.itemsize
+                       for part2dev_id in self._replica2part2dev_id)
+        else:
+            return self._dev_id_bytes
 
     @property
     def next_part_power(self):
+        if time() > self._rtime:
+            self._reload()
         return self._next_part_power
 
     @property
     def part_power(self):
         return 32 - self._part_shift
 
+    @property
+    def version(self):
+        return self._version
+
+    @property
+    def size(self):
+        return self._size
+
+    @property
+    def raw_size(self):
+        return self._raw_size
+
     def _rebuild_tier_data(self):
         self.tier2devs = defaultdict(list)
         for dev in self._devs:
@@ -300,6 +453,21 @@ class Ring(object):
         """Number of partitions in the ring."""
         return len(self._replica2part2dev_id[0])
 
+    @property
+    def device_count(self):
+        """Number of devices in the ring."""
+        return self._num_devs
+
+    @property
+    def weighted_device_count(self):
+        """Number of devices with weight in the ring."""
+        return self._num_weighted_devs
+
+    @property
+    def assigned_device_count(self):
+        """Number of devices with assignments in the ring."""
+        return self._num_assigned_devs
+
     @property
     def devs(self):
         """devices in the ring"""
@@ -407,15 +575,16 @@ class Ring(object):
         if time() > self._rtime:
             self._reload()
         primary_nodes = self._get_part_nodes(part)
-
         used = set(d['id'] for d in primary_nodes)
+        index = count()
         same_regions = set(d['region'] for d in primary_nodes)
         same_zones = set((d['region'], d['zone']) for d in primary_nodes)
         same_ips = set(
             (d['region'], d['zone'], d['ip']) for d in primary_nodes)
 
         parts = len(self._replica2part2dev_id[0])
-        part_hash = md5(str(part).encode('ascii')).digest()
+        part_hash = md5(str(part).encode('ascii'),
+                        usedforsecurity=False).digest()
         start = struct.unpack_from('>I', part_hash)[0] >> self._part_shift
         inc = int(parts / 65536) or 1
         # Multiple loops for execution speed; the checks and bookkeeping get
@@ -434,7 +603,7 @@ class Ring(object):
                     dev = self._devs[dev_id]
                     region = dev['region']
                     if dev_id not in used and region not in same_regions:
-                        yield dev
+                        yield dict(dev, handoff_index=next(index))
                         used.add(dev_id)
                         same_regions.add(region)
                         zone = dev['zone']
@@ -459,7 +628,7 @@ class Ring(object):
                     dev = self._devs[dev_id]
                     zone = (dev['region'], dev['zone'])
                     if dev_id not in used and zone not in same_zones:
-                        yield dev
+                        yield dict(dev, handoff_index=next(index))
                         used.add(dev_id)
                         same_zones.add(zone)
                         ip = zone + (dev['ip'],)
@@ -482,14 +651,14 @@ class Ring(object):
                     dev = self._devs[dev_id]
                     ip = (dev['region'], dev['zone'], dev['ip'])
                     if dev_id not in used and ip not in same_ips:
-                        yield dev
+                        yield dict(dev, handoff_index=next(index))
                         used.add(dev_id)
                         same_ips.add(ip)
                         if len(same_ips) == self._num_ips:
                             hit_all_ips = True
                             break
 
-        hit_all_devs = len(used) == self._num_devs
+        hit_all_devs = len(used) == self._num_assigned_devs
         for handoff_part in chain(range(start, parts, inc),
                                   range(inc - ((parts - start) % inc),
                                         start, inc)):
@@ -501,8 +670,9 @@ class Ring(object):
                 if handoff_part < len(part2dev_id):
                     dev_id = part2dev_id[handoff_part]
                     if dev_id not in used:
-                        yield self._devs[dev_id]
+                        dev = self._devs[dev_id]
+                        yield dict(dev, handoff_index=next(index))
                         used.add(dev_id)
-                        if len(used) == self._num_devs:
+                        if len(used) == self._num_assigned_devs:
                             hit_all_devs = True
                             break
