@@ -18,15 +18,17 @@ import base64
 import email.parser
 import itertools
 import json
+import time
 from copy import deepcopy
+from unittest import SkipTest
 
-import six
-
+import urllib
 from swift.common.swob import normalize_etag
-from swift.common.utils import md5
+from swift.common.utils import md5, config_true_value
 
+from test.functional import check_response, retry
 import test.functional as tf
-from test.functional import cluster_info, SkipTest
+from test.functional import cluster_info
 from test.functional.tests import Utils, Base, Base2, BaseEnv
 from test.functional.swift_test_client import Connection, ResponseError
 
@@ -49,6 +51,12 @@ def group_file_contents(file_contents):
         for char, grp in itertools.groupby(byte_iter)]
 
 
+def md5hex(s):
+    if not isinstance(s, bytes):
+        s = s.encode('ascii')
+    return md5(s, usedforsecurity=False).hexdigest()
+
+
 class TestSloEnv(BaseEnv):
     slo_enabled = None  # tri-state: None initially, then True/False
 
@@ -291,6 +299,338 @@ class TestSlo(Base):
             (b'e', 1),
         ], group_file_contents(file_contents))
 
+    def test_slo_multipart_delete_part_number_ignored(self):
+        # create a container just for this test because we're going to delete
+        # objects that we create
+        container = self.env.account.container(Utils.create_name())
+        self.assertTrue(container.create())
+        # create segments in same container
+        seg_info = self.env.create_segments(container)
+        file_item = container.file("manifest-abcde")
+        self.assertTrue(file_item.write(
+            json.dumps([seg_info['seg_a'], seg_info['seg_b'],
+                        seg_info['seg_c'], seg_info['seg_d'],
+                        seg_info['seg_e']]).encode('ascii'),
+            parms={'multipart-manifest': 'put'}))
+        # sanity check, we have SLO...
+        file_item.initialize(parms={'part-number': '5'})
+        self.assertEqual(
+            file_item.conn.response.getheader('X-Static-Large-Object'), 'True')
+        self.assertEqual(
+            file_item.conn.response.getheader('X-Parts-Count'), '5')
+        self.assertEqual(6, len(container.files()))
+
+        # part-number should be ignored
+        status = file_item.conn.make_request(
+            'DELETE', file_item.path,
+            parms={'multipart-manifest': 'delete',
+                   'part-number': '2'})
+        self.assertEqual(200, status)
+        # everything is gone
+        self.assertFalse(container.files())
+
+    def test_get_head_part_number_invalid(self):
+        file_item = self.env.container.file('manifest-abcde')
+        file_item.initialize()
+        ok_resp = file_item.conn.response
+        self.assertEqual(ok_resp.getheader('X-Static-Large-Object'), 'True')
+        self.assertEqual(ok_resp.getheader('Etag'), file_item.etag)
+        self.assertEqual(ok_resp.getheader('Content-Length'),
+                         str(file_item.size))
+
+        # part-number is 1-indexed
+        self.assertRaises(ResponseError, file_item.read,
+                          parms={'part-number': '0'})
+        resp_body = file_item.conn.response.read()
+        self.assertEqual(400, file_item.conn.response.status, resp_body)
+        self.assertEqual(b'Part number must be an integer greater than 0',
+                         resp_body)
+
+        self.assertRaises(ResponseError, file_item.initialize,
+                          parms={'part-number': '0'})
+        resp_body = file_item.conn.response.read()
+        self.assertEqual(400, file_item.conn.response.status)
+        self.assertEqual(b'', resp_body)
+
+    def test_get_head_part_number_out_of_range(self):
+        file_item = self.env.container.file('manifest-abcde')
+        file_item.initialize()
+        ok_resp = file_item.conn.response
+        self.assertEqual(ok_resp.getheader('X-Static-Large-Object'), 'True')
+        self.assertEqual(ok_resp.getheader('Etag'), file_item.etag)
+        self.assertEqual(ok_resp.getheader('Content-Length'),
+                         str(file_item.size))
+        manifest_etag = ok_resp.getheader('Manifest-Etag')
+
+        def check_headers(resp):
+            self.assertEqual(resp.getheader('X-Static-Large-Object'), 'True')
+            self.assertEqual(resp.getheader('Etag'), file_item.etag)
+            self.assertEqual(resp.getheader('Manifest-Etag'), manifest_etag)
+            self.assertEqual(resp.getheader('X-Parts-Count'), '5')
+            self.assertEqual(resp.getheader('Content-Range'),
+                             'bytes */%s' % file_item.size)
+
+        self.assertRaises(ResponseError, file_item.read,
+                          parms={'part-number': '10001'})
+        resp_body = file_item.conn.response.read()
+        self.assertEqual(416, file_item.conn.response.status, resp_body)
+        self.assertEqual(b'The requested part number is not satisfiable',
+                         resp_body)
+        check_headers(file_item.conn.response)
+        self.assertEqual(file_item.conn.response.getheader('Content-Length'),
+                         str(len(resp_body)))
+
+        self.assertRaises(ResponseError, file_item.info,
+                          parms={'part-number': '10001'})
+        resp_body = file_item.conn.response.read()
+        self.assertEqual(416, file_item.conn.response.status)
+        self.assertEqual(b'', resp_body)
+        check_headers(file_item.conn.response)
+        self.assertEqual(file_item.conn.response.getheader('Content-Length'),
+                         '0')
+
+    def test_get_part_number_simple_manifest(self):
+        file_item = self.env.container.file('manifest-abcde')
+        seg_info_list = [
+            self.env.seg_info["seg_%s" % letter]
+            for letter in ['a', 'b', 'c', 'd', 'e']
+        ]
+        checksum = md5(usedforsecurity=False)
+        total_size = 0
+        for seg_info in seg_info_list:
+            checksum.update(seg_info['etag'].encode('ascii'))
+            total_size += seg_info['size_bytes']
+        slo_etag = checksum.hexdigest()
+        start = 0
+        manifest_etag = None
+        for i, seg_info in enumerate(seg_info_list, start=1):
+            part_contents = file_item.read(parms={'part-number': i})
+            self.assertEqual(len(part_contents), seg_info['size_bytes'])
+            headers = dict(
+                (h.lower(), v)
+                for h, v in file_item.conn.response.getheaders())
+            self.assertEqual(headers['content-length'],
+                             str(seg_info['size_bytes']))
+            self.assertEqual(headers['etag'], '"%s"' % slo_etag)
+            if not manifest_etag:
+                manifest_etag = headers['x-manifest-etag']
+            else:
+                self.assertEqual(headers['x-manifest-etag'], manifest_etag)
+            end = start + seg_info['size_bytes'] - 1
+            self.assertEqual(headers['content-range'],
+                             'bytes %d-%d/%d' % (start, end, total_size), i)
+            self.assertEqual(headers['x-parts-count'], '5')
+            start = end + 1
+
+    def test_head_part_number_simple_manifest(self):
+        file_item = self.env.container.file('manifest-abcde')
+        seg_info_list = [
+            self.env.seg_info["seg_%s" % letter]
+            for letter in ['a', 'b', 'c', 'd', 'e']
+        ]
+        checksum = md5(usedforsecurity=False)
+        total_size = 0
+        for seg_info in seg_info_list:
+            checksum.update(seg_info['etag'].encode('ascii'))
+            total_size += seg_info['size_bytes']
+        slo_etag = checksum.hexdigest()
+        start = 0
+        manifest_etag = None
+        for i, seg_info in enumerate(seg_info_list, start=1):
+            part_info = file_item.info(parms={'part-number': i},
+                                       exp_status=206)
+            headers = dict(
+                (h.lower(), v)
+                for h, v in file_item.conn.response.getheaders())
+            self.assertEqual(headers['content-length'],
+                             str(seg_info['size_bytes']))
+            self.assertEqual(headers['etag'], '"%s"' % slo_etag)
+            self.assertEqual(headers['etag'], part_info['etag'])
+            if not manifest_etag:
+                manifest_etag = headers['x-manifest-etag']
+            else:
+                self.assertEqual(headers['x-manifest-etag'], manifest_etag)
+            end = start + seg_info['size_bytes'] - 1
+            self.assertEqual(headers['content-range'],
+                             'bytes %d-%d/%d' % (start, end, total_size), i)
+            self.assertEqual(headers['x-parts-count'], '5')
+            start = end + 1
+
+    def test_x_delete_at_with_part_number_and_open_expired(self):
+        cont_name = self.env.account.container(self.env.container.name)
+        allow_open_expired = config_true_value(tf.cluster_info['swift'].get(
+            'allow_open_expired', 'false'))
+
+        if not allow_open_expired:
+            raise SkipTest('allow_open_expired is disabled')
+
+        # data for segments
+        segments = [b'one', b'two', b'three', b'four']
+        etags = [md5hex(segment) for segment in segments]
+
+        def put_manifest(url, token, parsed, conn, object_segments,
+                         x_delete_after):
+            manifest_data = []
+            start = 0
+
+            for segment_object in range(len(object_segments)):
+                size = len(object_segments[segment_object])
+                end = start + size - 1
+                manifest_data.append({
+                    'path': '/%s/segments/%s' % (cont_name,
+                                                 str(segment_object)),
+                    'etag': etags[segment_object],
+                    'size_bytes': size,
+                })
+                start = end + 1
+
+            conn.request(
+                'PUT',
+                '%s/%s/manifest?multipart-manifest=put' % (parsed.path,
+                                                           cont_name),
+                body=json.dumps(manifest_data),
+                headers={
+                    'X-Auth-Token': token,
+                    'X-Delete-After': str(x_delete_after),
+                    'X-Static-Large-Object': 'true',
+                    'Content-Type': 'application/json'
+                }
+            )
+            resp = check_response(conn)
+            body = resp.read()
+            self.assertEqual(resp.status, 201,
+                             "Response status is not 201: %s" % body)
+
+        def put_segments(url, token, parsed, conn, object_segments,
+                         x_delete_after):
+            for objnum in range(len(object_segments)):
+                conn.request('PUT', '%s/%s/segments/%s' % (
+                    parsed.path,
+                    cont_name,
+                    str(objnum)),
+                    body=object_segments[objnum],
+                    headers={
+                        'X-Auth-Token': token,
+                        'X-Delete-After': str(x_delete_after)})
+                resp = check_response(conn)
+                body = resp.read()
+                self.assertEqual(resp.status, 201,
+                                 "Response status is not 201: %s" % body)
+
+        retry(put_segments, segments, 2)
+        retry(put_manifest, segments, 1)
+
+        # get the manifest
+        def get_manifest(url, token, parsed, conn, extra_headers=None):
+            headers = {'X-Auth-Token': token}
+            if extra_headers:
+                headers.update(extra_headers)
+            conn.request(
+                'GET',
+                '%s/%s/manifest?multipart-manifest=get' %
+                (parsed.path, cont_name),
+                '', headers)
+            return check_response(conn)
+
+        def get_manifest_still_expired(timeout=10, retry_delay=1):
+            start_time = time.time()
+            while time.time() - start_time < timeout:
+                resp = retry(get_manifest)
+                resp.read()
+                if resp.status == 404:
+                    break
+                elif 200 <= resp.status < 300:
+                    time.sleep(retry_delay)
+                else:
+                    self.fail(
+                        f'Unexpected response status {resp.status} for'
+                        f'manifest {resp.url}.')
+            else:
+                self.fail(
+                    f'manifest still returns {resp.status} after '
+                    f'we set x-delete-after to 2s post object '
+                    f'creation')
+
+            self.assertEqual(resp.status, 404,
+                             resp.headers.get('x-trans-id'))
+
+        get_manifest_still_expired()
+
+        def get_manifest_with_open_expire():
+            resp = retry(get_manifest, extra_headers={'X-Open-Expired': True})
+            self.assertEqual(resp.status, 200)
+            resp.read()
+
+        get_manifest_with_open_expire()
+
+        def get_or_head_part(url, token, parsed, conn,
+                             extra_headers=None, method='GET',
+                             part_number=None):
+            headers = {'X-Auth-Token': token}
+            if extra_headers:
+                headers.update(extra_headers)
+            conn.request(method, '%s/%s/manifest?part-number=%s' % (
+                parsed.path,
+                cont_name,
+                part_number
+            ), '', headers)
+            return check_response(conn)
+
+        resp = retry(get_manifest, extra_headers={'X-Open-Expired': True})
+        resp.read()
+        # read the expired object with magic x-open-expired header
+        self.assertEqual(resp.status, 200)
+
+        def check_parts_with_open_expire(method='GET'):
+            for objnum in range(len(segments)):
+                part_num = str(objnum + 1)
+                resp = retry(get_or_head_part,
+                             extra_headers={'X-Open-Expired': True},
+                             method=method, part_number=part_num)
+                resp.read()
+                self.assertEqual(resp.status, 206)
+
+        check_parts_with_open_expire()
+        check_parts_with_open_expire('HEAD')
+
+        def check_part_still_expired(method='GET', timeout=10, retry_delay=1):
+            for objnum, segment in enumerate(segments):
+                part_num = str(objnum + 1)
+                start_time = time.time()
+                while time.time() - start_time < timeout:
+                    resp = retry(get_or_head_part, method=method,
+                                 part_number=part_num)
+                    resp.read()
+                    if resp.status == 404:
+                        break
+                    elif 200 <= resp.status < 300:
+                        time.sleep(retry_delay)
+                    else:
+                        self.fail(
+                            f'Unexpected response status {resp.status} for'
+                            f' part {part_num}.')
+                else:
+                    self.fail(
+                        f'Part {part_num} did not return 404 within {timeout}'
+                        f' seconds.')
+
+        check_part_still_expired()
+        check_part_still_expired('HEAD', 5, 1)
+
+        def head_manifest(url, token, parsed, conn, extra_headers=None):
+            headers = {'X-Auth-Token': token}
+            if extra_headers:
+                headers.update(extra_headers)
+            conn.request('HEAD', '%s/%s/manifest' % (parsed.path,
+                                                     cont_name),
+                         '', headers)
+            return check_response(conn)
+
+        resp = retry(head_manifest, extra_headers={'X-Open-Expired': True})
+        resp.read()
+        # head expired object with magic x-open-expired header
+        self.assertEqual(resp.status, 200)
+
     def test_slo_container_listing(self):
         # the listing object size should equal the sum of the size of the
         # segments, not the size of the manifest body
@@ -458,10 +798,7 @@ class TestSlo(Base):
             hdrs={"Range": "bytes=1048571-1048580,2097147-2097156"})
 
         # See testMultiRangeGets for explanation
-        if six.PY2:
-            parser = email.parser.FeedParser()
-        else:
-            parser = email.parser.BytesFeedParser()
+        parser = email.parser.BytesFeedParser()
         parser.feed((
             "Content-Type: %s\r\n\r\n" % file_item.content_type).encode())
         parser.feed(file_contents)
@@ -707,8 +1044,44 @@ class TestSlo(Base):
         copied_contents = copied.read(parms={'multipart-manifest': 'get'})
         self.assertEqual(4 * 1024 * 1024 + 1, len(copied_contents))
 
+    def test_slo_copy_using_x_copy_from(self):
+        # as per test_slo_copy but using a PUT with x-copy-from
+        file_item = self.env.container.file("manifest-abcde")
+        file_item.copy_using_x_copy_from(
+            self.env.container.name, "copied-abcde")
+
+        copied = self.env.container.file("copied-abcde")
+        copied_contents = copied.read(parms={'multipart-manifest': 'get'})
+        self.assertEqual(4 * 1024 * 1024 + 1, len(copied_contents))
+
+    def test_slo_copy_part_number(self):
+        file_item = self.env.container.file("manifest-abcde")
+        file_item.copy(self.env.container.name, "copied-abcde",
+                       parms={'part-number': '1'})
+
+        copied = self.env.container.file("copied-abcde")
+        copied_contents = copied.read(parms={'multipart-manifest': 'get'})
+        # just the first part is copied
+        self.assertEqual(1024 * 1024, len(copied_contents))
+        self.assertEqual(b'a' * 10, copied_contents[:10])
+
+    def test_slo_copy_part_number_using_x_copy_from(self):
+        # as per test_slo_copy_part_number but using a PUT with x-copy-from
+        file_item = self.env.container.file("manifest-abcde")
+        # part-number on the client PUT target is actually applied to the
+        # internal GET source request
+        file_item.copy_using_x_copy_from(
+            self.env.container.name, "copied-abcde",
+            parms={'part-number': '1'})
+
+        copied = self.env.container.file("copied-abcde")
+        copied_contents = copied.read(parms={'multipart-manifest': 'get'})
+        # just the first part is copied
+        self.assertEqual(1024 * 1024, len(copied_contents))
+        self.assertEqual(b'a' * 10, copied_contents[:10])
+
     def test_slo_copy_account(self):
-        acct = self.env.conn.account_name
+        acct = urllib.parse.unquote(self.env.conn.account_name)
         # same account copy
         file_item = self.env.container.file("manifest-abcde")
         file_item.copy_account(acct, self.env.container.name, "copied-abcde")
@@ -719,7 +1092,7 @@ class TestSlo(Base):
 
         if not tf.skip2:
             # copy to different account
-            acct = self.env.conn2.account_name
+            acct = urllib.parse.unquote(self.env.conn2.account_name)
             dest_cont = self.env.account2.container(Utils.create_name())
             self.assertTrue(dest_cont.create(hdrs={
                 'X-Container-Write': self.env.conn.user_acl
@@ -805,6 +1178,38 @@ class TestSlo(Base):
         self.assertEqual(copied_json[0], {
             'data': base64.b64encode(b'APRE' * 8).decode('ascii')})
 
+    def test_slo_copy_the_manifest_using_x_copy_from(self):
+        # as per test_slo_copy_the_manifest but using a PUT with x-copy-from
+        source = self.env.container.file("manifest-abcde")
+        source.initialize(parms={'multipart-manifest': 'get'})
+        source_contents = source.read(parms={'multipart-manifest': 'get'})
+        source_json = json.loads(source_contents)
+        manifest_etag = md5(source_contents, usedforsecurity=False).hexdigest()
+        if tf.cluster_info.get('etag_quoter', {}).get('enable_by_default'):
+            manifest_etag = '"%s"' % manifest_etag
+        self.assertEqual(manifest_etag, source.etag)
+
+        source.initialize()
+        self.assertEqual('application/octet-stream', source.content_type)
+        self.assertNotEqual(manifest_etag, source.etag)
+
+        # multipart-manifest=get on the client PUT target request actually
+        # applies to the internal GET source request
+        self.assertTrue(
+            source.copy_using_x_copy_from(self.env.container.name,
+                                          "copied-abcde-manifest-only",
+                                          parms={'multipart-manifest': 'get'}))
+
+        copied = self.env.container.file("copied-abcde-manifest-only")
+        copied.initialize(parms={'multipart-manifest': 'get'})
+        copied_contents = copied.read(parms={'multipart-manifest': 'get'})
+        try:
+            copied_json = json.loads(copied_contents)
+        except ValueError:
+            self.fail("COPY didn't copy the manifest (invalid json on GET)")
+        self.assertEqual(source_json, copied_json)
+        self.assertEqual(manifest_etag, copied.etag)
+
     def test_slo_copy_the_manifest_updating_metadata(self):
         source = self.env.container.file("manifest-abcde")
         source.content_type = 'application/octet-stream'
@@ -870,7 +1275,7 @@ class TestSlo(Base):
     def test_slo_copy_the_manifest_account(self):
         if tf.skip2:
             raise SkipTest('Account2 not set')
-        acct = self.env.conn.account_name
+        acct = urllib.parse.unquote(self.env.conn.account_name)
         # same account
         file_item = self.env.container.file("manifest-abcde")
         file_item.copy_account(acct,
@@ -886,7 +1291,7 @@ class TestSlo(Base):
             self.fail("COPY didn't copy the manifest (invalid json on GET)")
 
         # different account
-        acct = self.env.conn2.account_name
+        acct = urllib.parse.unquote(self.env.conn2.account_name)
         dest_cont = self.env.account2.container(Utils.create_name())
         self.assertTrue(dest_cont.create(hdrs={
             'X-Container-Write': self.env.conn.user_acl
@@ -1109,8 +1514,6 @@ class TestSlo(Base):
             value[0]['hash'],
             md5(b'd' * 1024 * 1024, usedforsecurity=False).hexdigest())
         expected_name = '/%s/seg_d' % self.env.container.name
-        if six.PY2:
-            expected_name = expected_name.decode("utf-8")
         self.assertEqual(value[0]['name'], expected_name)
 
         self.assertEqual(value[1]['bytes'], 1024 * 1024)
@@ -1118,8 +1521,6 @@ class TestSlo(Base):
             value[1]['hash'],
             md5(b'b' * 1024 * 1024, usedforsecurity=False).hexdigest())
         expected_name = '/%s/seg_b' % self.env.container.name
-        if six.PY2:
-            expected_name = expected_name.decode("utf-8")
         self.assertEqual(value[1]['name'], expected_name)
 
     def test_slo_get_raw_the_manifest_with_details_from_server(self):
@@ -1145,16 +1546,12 @@ class TestSlo(Base):
             value[0]['etag'],
             md5(b'd' * 1024 * 1024, usedforsecurity=False).hexdigest())
         expected_name = '/%s/seg_d' % self.env.container.name
-        if six.PY2:
-            expected_name = expected_name.decode("utf-8")
         self.assertEqual(value[0]['path'], expected_name)
         self.assertEqual(value[1]['size_bytes'], 1024 * 1024)
         self.assertEqual(
             value[1]['etag'],
             md5(b'b' * 1024 * 1024, usedforsecurity=False).hexdigest())
         expected_name = '/%s/seg_b' % self.env.container.name
-        if six.PY2:
-            expected_name = expected_name.decode("utf-8")
         self.assertEqual(value[1]['path'], expected_name)
 
         file_item = self.env.container.file("manifest-from-get-raw")
