@@ -470,7 +470,7 @@ class TestReaper(unittest.TestCase):
                       self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
         self.assertEqual(
-            r.logger.statsd_client.get_increment_counts()['return_codes.4'], 1)
+            r.logger.statsd_client.get_stats_counts()['return_codes.4'], 1)
         self.assertEqual(r.stats_containers_deleted, 1)
 
     def test_reap_container_partial_fail(self):
@@ -490,7 +490,7 @@ class TestReaper(unittest.TestCase):
                       self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
         self.assertEqual(
-            r.logger.statsd_client.get_increment_counts()['return_codes.4'], 4)
+            r.logger.statsd_client.get_stats_counts()['return_codes.4'], 4)
         self.assertEqual(r.stats_containers_possibly_remaining, 1)
 
     def test_reap_container_full_fail(self):
@@ -510,7 +510,7 @@ class TestReaper(unittest.TestCase):
                       self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
         self.assertEqual(
-            r.logger.statsd_client.get_increment_counts()['return_codes.4'], 5)
+            r.logger.statsd_client.get_stats_counts()['return_codes.4'], 5)
         self.assertEqual(r.stats_containers_remaining, 1)
 
     def test_reap_container_get_object_timeout(self):
