@@ -15,9 +15,10 @@
 
 import unittest
 
-from swift.common.swob import Response
+from swift.common.swob import Response, Request
 from swift.common.utils import HeaderKeyDict
-from swift.common.middleware.s3api.s3response import S3Response
+from swift.common.middleware.catch_errors import CatchErrorMiddleware
+from swift.common.middleware.s3api.s3response import S3Response, ErrorResponse
 from swift.common.middleware.s3api.utils import sysmeta_prefix
 
 
@@ -103,5 +104,48 @@ class TestResponse(unittest.TestCase):
             self.assertEqual(expected_headers, s3resp.sysmeta_headers)
 
 
+class DummyErrorResponse(ErrorResponse):
+    _status = "418 I'm a teapot"
+
+
+class TestErrorResponse(unittest.TestCase):
+    def test_error_response(self):
+        resp = DummyErrorResponse(msg='my-msg', reason='my reason')
+        self.assertEqual("418 I'm a teapot", str(resp))
+        self.assertEqual("418 I'm a teapot", resp.status)
+        self.assertEqual(418, resp.status_int)
+        self.assertEqual('my reason', resp.reason)
+        self.assertEqual('DummyErrorResponse.my_reason', resp.summary)
+        self.assertEqual('418.DummyErrorResponse.my_reason', resp.metric_name)
+        self.assertEqual(
+            b"<?xml version='1.0' encoding='UTF-8'?>\n"
+            b"<Error>"
+            b"<Code>DummyErrorResponse</Code>"
+            b"<Message>my-msg</Message>"
+            b"</Error>",
+            resp.body)
+
+    def test_error_response_trans_id(self):
+        req = Request.blank('/bucket/object')
+        err = DummyErrorResponse(msg='my-msg', reason='my reason')
+        app = CatchErrorMiddleware(err, {})
+        with unittest.mock.patch(
+                'swift.common.middleware.catch_errors.generate_trans_id',
+                return_value='fake-trans-id'):
+            resp = req.get_response(app)
+        self.assertIn('swift.trans_id', req.environ)
+        self.assertEqual(418, resp.status_int)
+        self.assertIn('X-Trans-Id', resp.headers)
+        self.assertEqual(
+            b"<?xml version='1.0' encoding='UTF-8'?>\n"
+            b"<Error>"
+            b"<Code>DummyErrorResponse</Code>"
+            b"<Message>my-msg</Message>"
+            b"<RequestId>fake-trans-id</RequestId>"
+            b"</Error>",
+            resp.body)
+        self.assertEqual(146, int(resp.headers['Content-Length']))
+
+
 if __name__ == '__main__':
     unittest.main()
