@@ -21,16 +21,13 @@ py34 unit test job and there are imports here that end up importing modules
 that are not yet ported to py34, such wsgi.py which import mimetools.
 """
 import os
-from contextlib import closing
-from gzip import GzipFile
 from tempfile import mkdtemp
 import time
-
+import warnings
 
 from eventlet import spawn, wsgi
-import mock
+from unittest import mock
 from shutil import rmtree
-import six.moves.cPickle as pickle
 
 import swift
 from swift.account import server as account_server
@@ -64,8 +61,8 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
     :returns: A dict containing the following entries:
                   orig_POLICIES: the value of storage_policy.POLICIES prior to
                                  it being patched with fake policies
-                  orig_SysLogHandler: the value of utils.SysLogHandler prior to
-                                      it being patched
+                  orig_SysLogHandler: the value of utils.logs.SysLogHandler
+                                      prior to it being patched
                   testdir: root directory used for test files
                   test_POLICIES: a StoragePolicyCollection of fake policies
                   test_servers: a tuple of test server instances
@@ -75,10 +72,10 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
     """
     context = {
         "orig_POLICIES": storage_policy._POLICIES,
-        "orig_SysLogHandler": utils.SysLogHandler}
+        "orig_SysLogHandler": utils.logs.SysLogHandler}
 
     utils.HASH_PATH_SUFFIX = b'endcap'
-    utils.SysLogHandler = mock.MagicMock()
+    utils.logs.SysLogHandler = mock.MagicMock()
     # Since we're starting up a lot here, we're going to test more than
     # just chunked puts; we're also going to test parts of
     # proxy_server.Application we couldn't get to easily otherwise.
@@ -96,6 +93,7 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
             'allow_versions': 't', 'node_timeout': 20}
     if extra_conf:
         conf.update(extra_conf)
+    context['conf'] = conf
     prolis = listen_zero()
     acc1lis = listen_zero()
     acc2lis = listen_zero()
@@ -149,18 +147,18 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
     # write_fake_ring can't handle a 3-element ring, and the EC policy needs
     # at least 6 devs to work with (ec_k=2, ec_m=1, duplication_factor=2),
     # so we do it manually
-    devs = [{'id': 0, 'zone': 0, 'device': 'sdg1', 'ip': '127.0.0.1',
-             'port': obj1lis.getsockname()[1]},
-            {'id': 1, 'zone': 0, 'device': 'sdh1', 'ip': '127.0.0.1',
-             'port': obj2lis.getsockname()[1]},
-            {'id': 2, 'zone': 0, 'device': 'sdi1', 'ip': '127.0.0.1',
-             'port': obj3lis.getsockname()[1]},
-            {'id': 3, 'zone': 0, 'device': 'sdj1', 'ip': '127.0.0.1',
-             'port': obj4lis.getsockname()[1]},
-            {'id': 4, 'zone': 0, 'device': 'sdk1', 'ip': '127.0.0.1',
-             'port': obj5lis.getsockname()[1]},
-            {'id': 5, 'zone': 0, 'device': 'sdl1', 'ip': '127.0.0.1',
-             'port': obj6lis.getsockname()[1]}]
+    devs = [{'id': 0, 'region': 1, 'zone': 0, 'device': 'sdg1',
+             'ip': '127.0.0.1', 'port': obj1lis.getsockname()[1]},
+            {'id': 1, 'region': 1, 'zone': 0, 'device': 'sdh1',
+             'ip': '127.0.0.1', 'port': obj2lis.getsockname()[1]},
+            {'id': 2, 'region': 1, 'zone': 0, 'device': 'sdi1',
+             'ip': '127.0.0.1', 'port': obj3lis.getsockname()[1]},
+            {'id': 3, 'region': 1, 'zone': 0, 'device': 'sdj1',
+             'ip': '127.0.0.1', 'port': obj4lis.getsockname()[1]},
+            {'id': 4, 'region': 1, 'zone': 0, 'device': 'sdk1',
+             'ip': '127.0.0.1', 'port': obj5lis.getsockname()[1]},
+            {'id': 5, 'region': 1, 'zone': 0, 'device': 'sdl1',
+             'ip': '127.0.0.1', 'port': obj6lis.getsockname()[1]}]
     pol3_replica2part2dev_id = [[0, 1, 2, 0],
                                 [1, 2, 0, 1],
                                 [2, 0, 1, 2]]
@@ -173,14 +171,12 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
     obj3_ring_path = os.path.join(
         _testdir, storage_policy.POLICIES[3].ring_name + '.ring.gz')
     part_shift = 30
-    with closing(GzipFile(obj3_ring_path, 'wb')) as fh:
-        pickle.dump(RingData(pol3_replica2part2dev_id, devs, part_shift), fh)
+    RingData(pol3_replica2part2dev_id, devs, part_shift).save(obj3_ring_path)
 
     obj4_ring_path = os.path.join(
         _testdir, storage_policy.POLICIES[4].ring_name + '.ring.gz')
     part_shift = 30
-    with closing(GzipFile(obj4_ring_path, 'wb')) as fh:
-        pickle.dump(RingData(pol4_replica2part2dev_id, devs, part_shift), fh)
+    RingData(pol4_replica2part2dev_id, devs, part_shift).save(obj4_ring_path)
 
     prosrv = proxy_server.Application(conf, logger=debug_logger('proxy'))
     for policy in storage_policy.POLICIES:
@@ -215,6 +211,10 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
     logging_prosv = proxy_logging.ProxyLoggingMiddleware(
         listing_formats.ListingFilter(prosrv, {}, logger=prosrv.logger),
         conf, logger=prosrv.logger)
+    # Yes, eventlet, we know -- we have to support bad clients, though
+    warnings.filterwarnings(
+        'ignore', module='eventlet',
+        message='capitalize_response_headers is disabled')
     prospa = spawn(wsgi.server, prolis, logging_prosv, nl,
                    protocol=SwiftHttpProtocol,
                    capitalize_response_headers=False)
@@ -262,7 +262,7 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
                                                         {'X-Timestamp': ts,
                                                          'x-trans-id': 'test'})
         resp = conn.getresponse()
-        assert(resp.status == 201)
+        assert resp.status == 201
     # Create another account
     # used for account-to-account tests
     ts = normalize_timestamp(time.time())
@@ -276,7 +276,7 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
                                                         {'X-Timestamp': ts,
                                                          'x-trans-id': 'test'})
         resp = conn.getresponse()
-        assert(resp.status == 201)
+        assert resp.status == 201
     # Create containers, 1 per test policy
     sock = connect_tcp(('localhost', prolis.getsockname()[1]))
     fd = sock.makefile('rwb')
@@ -330,6 +330,10 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
 def teardown_servers(context):
     for server in context["test_coros"]:
         server.kill()
+    # We didn't start the proxy w/ run_server, so we have to kill the
+    # watchdog ourselves
+    context["test_servers"][0].watchdog.kill()
+    assert context["test_servers"][0].watchdog._run_gth is None
     rmtree(os.path.dirname(context["testdir"]))
-    utils.SysLogHandler = context["orig_SysLogHandler"]
+    utils.logs.SysLogHandler = context["orig_SysLogHandler"]
     storage_policy._POLICIES = context["orig_POLICIES"]
