@@ -14,12 +14,12 @@
 # limitations under the License.
 from collections import defaultdict
 
-import mock
+from unittest import mock
 import os
 import unittest
 
 import eventlet
-from six.moves import urllib
+import urllib.parse
 
 from swift.common.exceptions import DiskFileNotExist, DiskFileError, \
     DiskFileDeleted, DiskFileExpired
@@ -62,8 +62,10 @@ class TestBaseSsync(BaseTest):
         self.ts_iter = make_timestamp_iter()
         self.rx_ip = '127.0.0.1'
         sock = listen_zero()
+        self.rx_server_pool = eventlet.GreenPool(size=1)
         self.rx_server = eventlet.spawn(
-            eventlet.wsgi.server, sock, self.rx_controller, self.rx_logger)
+            eventlet.wsgi.server, sock, self.rx_controller, log=self.rx_logger,
+            custom_pool=self.rx_server_pool)
         self.rx_port = sock.getsockname()[1]
         self.rx_node = {'replication_ip': self.rx_ip,
                         'replication_port': self.rx_port,
@@ -74,6 +76,15 @@ class TestBaseSsync(BaseTest):
         self.rx_server.kill()
         super(TestBaseSsync, self).tearDown()
 
+    def _wait_for_rx_server(self):
+        # wait for receiver thread to complete before checking logs, but don't
+        # wait forever
+        with eventlet.Timeout(
+                seconds=1,
+                exception=AssertionError(
+                    'timed out waiting for ssync receiver thread')):
+            self.rx_server_pool.waitall()
+
     def make_connect_wrapper(self, sender):
         """
         Make a wrapper function for the ssync_sender.Sender.connect() method
@@ -129,6 +140,7 @@ class TestBaseSsync(BaseTest):
                                                 frag_index=frag_index)
             if policy.policy_type == EC_POLICY:
                 metadata['X-Object-Sysmeta-Ec-Frag-Index'] = str(frag_index)
+                metadata['X-Object-Sysmeta-Ec-Etag'] = 'fake-etag'
             df = self._make_diskfile(
                 device=self.device, partition=self.partition, account='a',
                 container='c', obj=obj_name, body=object_data,
@@ -990,18 +1002,21 @@ class TestSsyncECReconstructorSyncJob(TestBaseSsyncEC):
                 pass  # expected outcome
         if msgs:
             self.fail('Failed with:\n%s' % '\n'.join(msgs))
-        log_lines = self.logger.get_lines_for_level('error')
+        tx_error_log_lines = self.logger.get_lines_for_level('error')
         self.assertIn('Sent data length does not match content-length',
-                      log_lines[0])
-        self.assertFalse(log_lines[1:])
-        # trampoline for the receiver to write a log
-        eventlet.sleep(0)
-        log_lines = self.rx_logger.get_lines_for_level('warning')
-        self.assertEqual(1, len(log_lines), self.rx_logger.all_log_lines())
+                      tx_error_log_lines[0])
+        self.assertFalse(tx_error_log_lines[1:])
+        self._wait_for_rx_server()
+        rx_warning_log_lines = self.rx_logger.get_lines_for_level('warning')
+        self.assertEqual(1, len(rx_warning_log_lines),
+                         self.rx_logger.all_log_lines())
         self.assertIn('ssync subrequest failed with 499',
-                      log_lines[0])
-        self.assertFalse(log_lines[1:])
-        self.assertFalse(self.rx_logger.get_lines_for_level('error'))
+                      rx_warning_log_lines[0])
+        self.assertFalse(rx_warning_log_lines[1:])
+        rx_error_lines = self.rx_logger.get_lines_for_level('error')
+        self.assertEqual(1, len(rx_error_lines), rx_error_lines)
+        self.assertIn('127.0.0.1/dev/9 read failed in ssync.Receiver: Early '
+                      'termination for PUT', rx_error_lines[0])
 
     def test_sync_reconstructor_no_rebuilt_content(self):
         # First fragment to sync gets no content in any response to
@@ -1025,17 +1040,19 @@ class TestSsyncECReconstructorSyncJob(TestBaseSsyncEC):
                 pass  # expected outcome
         if msgs:
             self.fail('Failed with:\n%s' % '\n'.join(msgs))
-        log_lines = self.logger.get_lines_for_level('error')
+        tx_error_log_lines = self.logger.get_lines_for_level('error')
         self.assertIn('Sent data length does not match content-length',
-                      log_lines[0])
-        self.assertFalse(log_lines[1:])
-        # trampoline for the receiver to write a log
-        eventlet.sleep(0)
-        log_lines = self.rx_logger.get_lines_for_level('warning')
+                      tx_error_log_lines[0])
+        self.assertFalse(tx_error_log_lines[1:])
+        self._wait_for_rx_server()
+        rx_warning_log_lines = self.rx_logger.get_lines_for_level('warning')
         self.assertIn('ssync subrequest failed with 499',
-                      log_lines[0])
-        self.assertFalse(log_lines[1:])
-        self.assertFalse(self.rx_logger.get_lines_for_level('error'))
+                      rx_warning_log_lines[0])
+        self.assertFalse(rx_warning_log_lines[1:])
+        rx_error_lines = self.rx_logger.get_lines_for_level('error')
+        self.assertEqual(1, len(rx_error_lines), rx_error_lines)
+        self.assertIn('127.0.0.1/dev/9 read failed in ssync.Receiver: Early '
+                      'termination for PUT', rx_error_lines[0])
 
     def test_sync_reconstructor_exception_during_rebuild(self):
         # First fragment to sync has some reconstructor get responses raise
@@ -1070,18 +1087,20 @@ class TestSsyncECReconstructorSyncJob(TestBaseSsyncEC):
         if msgs:
             self.fail('Failed with:\n%s' % '\n'.join(msgs))
 
-        log_lines = self.logger.get_lines_for_level('error')
-        self.assertIn('Error trying to rebuild', log_lines[0])
+        tx_error_log_lines = self.logger.get_lines_for_level('error')
+        self.assertIn('Error trying to rebuild', tx_error_log_lines[0])
         self.assertIn('Sent data length does not match content-length',
-                      log_lines[1])
-        self.assertFalse(log_lines[2:])
-        # trampoline for the receiver to write a log
-        eventlet.sleep(0)
-        log_lines = self.rx_logger.get_lines_for_level('warning')
+                      tx_error_log_lines[1])
+        self.assertFalse(tx_error_log_lines[2:])
+        self._wait_for_rx_server()
+        rx_warning_log_lines = self.rx_logger.get_lines_for_level('warning')
         self.assertIn('ssync subrequest failed with 499',
-                      log_lines[0])
-        self.assertFalse(log_lines[1:])
-        self.assertFalse(self.rx_logger.get_lines_for_level('error'))
+                      rx_warning_log_lines[0])
+        self.assertFalse(rx_warning_log_lines[1:])
+        rx_error_lines = self.rx_logger.get_lines_for_level('error')
+        self.assertEqual(1, len(rx_error_lines), rx_error_lines)
+        self.assertIn('127.0.0.1/dev/9 read failed in ssync.Receiver: Early '
+                      'termination for PUT', rx_error_lines[0])
 
     def test_sync_reconstructor_no_responses(self):
         # First fragment to sync gets no responses for reconstructor to rebuild
@@ -1129,8 +1148,7 @@ class TestSsyncECReconstructorSyncJob(TestBaseSsyncEC):
             self.fail('Failed with:\n%s' % '\n'.join(msgs))
         log_lines = self.logger.get_lines_for_level('error')
         self.assertIn('Unable to get enough responses', log_lines[0])
-        # trampoline for the receiver to write a log
-        eventlet.sleep(0)
+        self._wait_for_rx_server()
         self.assertFalse(self.rx_logger.get_lines_for_level('warning'))
         self.assertFalse(self.rx_logger.get_lines_for_level('error'))
 
@@ -1232,8 +1250,7 @@ class TestSsyncECReconstructorSyncJob(TestBaseSsyncEC):
                                       frag_index=self.tx_node_index),
                 fd.read())
 
-        # trampoline for the receiver to write a log
-        eventlet.sleep(0)
+        self._wait_for_rx_server()
         self.assertFalse(self.rx_logger.get_lines_for_level('warning'))
         self.assertFalse(self.rx_logger.get_lines_for_level('error'))
 
@@ -1268,8 +1285,7 @@ class TestSsyncECReconstructorSyncJob(TestBaseSsyncEC):
         self.assertFalse(self.logger.get_lines_for_level('error'))
         self.assertFalse(
             self.logger.get_lines_for_level('error'))
-        # trampoline for the receiver to write a log
-        eventlet.sleep(0)
+        self._wait_for_rx_server()
         self.assertFalse(self.rx_logger.get_lines_for_level('warning'))
         self.assertFalse(self.rx_logger.get_lines_for_level('error'))
 
@@ -1436,19 +1452,21 @@ class TestSsyncReplication(TestBaseSsync):
         rx_objs['o2'] = self._create_ondisk_files(rx_df_mgr, 'o2', policy, t2)
         expected_subreqs['POST'].append('o2')
 
-        # o3 is on tx with meta, rx has newer data but no meta
+        # o3 is on tx with meta, rx has newer data but no meta,
+        # meta timestamp has an offset
         t3a = next(self.ts_iter)
         tx_objs['o3'] = self._create_ondisk_files(tx_df_mgr, 'o3', policy, t3a)
         t3b = next(self.ts_iter)
         rx_objs['o3'] = self._create_ondisk_files(rx_df_mgr, 'o3', policy, t3b)
         t3_meta = next(self.ts_iter)
+        t3_meta = utils.Timestamp(t3_meta, offset=2)
         metadata = {'X-Timestamp': t3_meta.internal,
                     'X-Object-Meta-Test': 'o3',
                     'X-Object-Sysmeta-Test': 'sys_o3'}
         tx_objs['o3'][0].write_metadata(metadata)
         expected_subreqs['POST'].append('o3')
 
-        # o4 is on tx with meta, rx has older data and up to date meta
+        # o4 is on tx with meta, rx has older data and up to date meta,
         t4a = next(self.ts_iter)
         rx_objs['o4'] = self._create_ondisk_files(rx_df_mgr, 'o4', policy, t4a)
         t4b = next(self.ts_iter)
@@ -1463,6 +1481,7 @@ class TestSsyncReplication(TestBaseSsync):
 
         # o5 is on tx with meta, rx is in sync with data and meta
         t5 = next(self.ts_iter)
+        t5 = utils.Timestamp(t5, offset=1)  # note: use an offset for this test
         rx_objs['o5'] = self._create_ondisk_files(rx_df_mgr, 'o5', policy, t5)
         tx_objs['o5'] = self._create_ondisk_files(tx_df_mgr, 'o5', policy, t5)
         t5_meta = next(self.ts_iter)
@@ -1497,6 +1516,25 @@ class TestSsyncReplication(TestBaseSsync):
         tx_objs['o7'][0].write_metadata(metadata)
         rx_tombstones['o7'][0].delete(next(self.ts_iter))
 
+        # o8 is on tx with meta, rx has in sync data but meta with different
+        # offset
+        t8 = next(self.ts_iter)
+        rx_objs['o8'] = self._create_ondisk_files(rx_df_mgr, 'o8', policy, t8)
+        tx_objs['o8'] = self._create_ondisk_files(tx_df_mgr, 'o8', policy, t8)
+        t8_meta = next(self.ts_iter)
+        t8_meta_offset = utils.Timestamp(t8_meta, offset=4)
+        metadata = {'X-Timestamp': t8_meta_offset.internal,
+                    'X-Object-Meta-Test': 'o8',
+                    'X-Object-Sysmeta-Test': 'sys_o8'}
+        tx_objs['o8'][0].write_metadata(metadata)
+        # different ts_meta offset on rx
+        t8_meta_offset = utils.Timestamp(t8_meta, offset=3)
+        metadata = {'X-Timestamp': t8_meta_offset.internal,
+                    'X-Object-Meta-Test': 'o8',
+                    'X-Object-Sysmeta-Test': 'sys_o8'}
+        rx_objs['o8'][0].write_metadata(metadata)
+        expected_subreqs['POST'].append('o8')
+
         suffixes = set()
         for diskfiles in list(tx_objs.values()) + list(tx_tombstones.values()):
             for df in diskfiles:
@@ -1514,13 +1552,13 @@ class TestSsyncReplication(TestBaseSsync):
         # run the sync protocol...
         success, in_sync_objs = sender()
 
-        self.assertEqual(7, len(in_sync_objs))
+        self.assertEqual(8, len(in_sync_objs))
         self.assertTrue(success)
 
         # verify protocol
         results = self._analyze_trace(trace)
-        self.assertEqual(7, len(results['tx_missing']))
-        self.assertEqual(5, len(results['rx_missing']))
+        self.assertEqual(8, len(results['tx_missing']))
+        self.assertEqual(6, len(results['rx_missing']))
         for subreq in results.get('tx_updates'):
             obj = subreq['path'].split('/')[3]
             method = subreq['method']
